# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-08-02 20:54
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shorty', '0003_auto_20160602_1131'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shorturl',
            name='external',
            field=models.BooleanField(default=False, help_text=b"Should this short link be available outside the company's network?"),
        ),
        migrations.AlterField(
            model_name='shorturl',
            name='redirect',
            field=models.TextField(validators=[django.core.validators.URLValidator()]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-06-02 11:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('shorty', '0002_auto_20160322_1720'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='shorturl',
            options={'ordering': ('-created',), 'verbose_name': 'Short URL', 'verbose_name_plural': 'Short URLs'},
        ),
        migrations.AddField(
            model_name='shorturl',
            name='external',
            field=models.BooleanField(default=False, help_text=b'Available externally'),
        ),
        migrations.AlterField(
            model_name='shorturl',
            name='path',
            field=models.SlugField(error_messages={b'unique': b'This short URL is already in use; please choose something different, or leave blank for a random URL'}, unique=True),
        ),
    ]



'''Shorty Tests'''

from django.test import TestCase, Client
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

import mock

from .models import ShortURL
from .forms import ShortURLForm
from . import views


class UserTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user('test', 'test@test.com', 'test')
        super(UserTestCase, self).setUp()


class ModelTestCase(UserTestCase):

    def test_random_slug_generation(self):
        one = ShortURL.objects.create(redirect='http://www.google.com', user=self.user)
        two = ShortURL.objects.create(redirect='http://www.foo.com', user=self.user)
        self.assertNotEqual(one.path, two.path)
        self.assertNotEqual(one.path, '')
        self.assertNotEqual(two.path, '')


class RedirectViewTestCase(UserTestCase):

    def setUp(self):
        super(RedirectViewTestCase, self).setUp()
        ShortURL.objects.create(redirect='http://www.google.com', path='google', user=self.user, external=True)
        ShortURL.objects.create(redirect='http://www.foo.com', path='foo', user=self.user)
        self.client = Client()

    def test_missing_slug(self):
        '''Missing slugs should raise a 404'''
        response = self.client.get('/404')
        self.assertEqual(response.status_code, 404)

    def test_no_slug(self):
        '''No slug at all should redirect to admin'''
        response = self.client.get('/', HTTP_HOST='shorty.example.com')
        self.assertRedirects(response, '/admin/', host='shorty.example.com', fetch_redirect_response=False)

    def test_no_slug_no_host_header(self):
        '''No slug (and no Host header) should return a 400 Bad Request'''
        response = self.client.get('/')
        self.assertEqual(response.status_code, 400)

    def test_unauthenticated_redirect_canonical_false_external_false(self):
        '''Unauthenticated clients - no settings: should simply get redirects'''
        response = self.client.get('/google')
        self.assertRedirects(response, 'http://www.google.com', fetch_redirect_response=False)
        response = self.client.get('/foo/')
        self.assertRedirects(response, 'http://www.foo.com', fetch_redirect_response=False)

    @mock.patch.object(views, 'EXTERNAL_FLAG', True)
    def test_unauthenticated_redirect_canonical_false_external_true(self):
        '''Unauthenticated clients, external access'''
        with self.settings(LOGIN_URL='/login/'):
            # External=True objects should return redirect
            response = self.client.get('/google')
            self.assertRedirects(response, 'http://www.google.com', fetch_redirect_response=False)
            # External=False objects should force login
            response = self.client.get('/foo/')
            self.assertRedirects(response, '/login/?next=/foo/', fetch_redirect_response=False)

    @mock.patch.object(views, 'CANONICAL_DOMAIN', 'http://shorty.example.com')
    def test_unauthenticated_redirect_canonical_true_external_false(self):
        '''Unauthenticated clients, no external access, canonical domain'''
        # Matching domain and scheme should redirect like usual
        response = self.client.get('/google', HTTP_HOST='shorty.example.com')
        self.assertRedirects(response, 'http://www.google.com', fetch_redirect_response=False)
        # Not matching domain should redirect to matching domain
        response = self.client.get('/google', HTTP_HOST='shorty1.example.com')
        self.assertRedirects(response, 'http://shorty.example.com/google', fetch_redirect_response=False)

    @mock.patch.object(views, 'CANONICAL_DOMAIN', 'https://shorty.example.com')
    def test_unauthenticated_redirect_canonical_https_external_false(self):
        '''Unauthenticated clients, no external access, canonical domain'''
        # Non matching schcme should redirect to matching
        response = self.client.get('/google', HTTP_HOST='shorty.example.com')
        self.assertRedirects(response, 'https://shorty.example.com/google', fetch_redirect_response=False)

    @mock.patch.object(views, 'EXTERNAL_FLAG', True)
    def test_authenticated_redirect_canonical_false_external_true(self):
        '''Authenticated clients, external access, redirects should work'''
        self.client.login(username='test', password='test')
        response = self.client.get('/foo')
        self.assertRedirects(response, 'http://www.foo.com', fetch_redirect_response=False)

    def test_preview(self):
        '''Test preview domain'''
        response = self.client.get('/google?preview=True')
        self.assertContains(response, "<a href='/google'>http://www.google.com</a>", html=True)
        response = self.client.get('/google?p=True')
        self.assertContains(response, "<a href='/google'>http://www.google.com</a>", html=True)

    @mock.patch.object(views, 'CANONICAL_DOMAIN', 'http://shorty.example.com')
    def test_preview_preseved_over_canonical_redirect(self):
        '''Unauthenticated clients, no external access, canonical domain'''
        # Not matching domain should redirect to matching domain
        response = self.client.get('/google?preview=True', HTTP_HOST='shorty1.example.com')
        self.assertRedirects(response, 'http://shorty.example.com/google?preview=True', fetch_redirect_response=False)


class AuthenticatedUITestCase(UserTestCase):

    def setUp(self):
        super(AuthenticatedUITestCase, self).setUp()
        ShortURL.objects.create(redirect='http://www.google.com', path='google', user=self.user, external=True)
        ShortURL.objects.create(redirect='http://www.foo.com', path='foo', user=self.user)
        self.client = Client()
        self.client.login(username='test', password='test')

    def test_delete_short_url(self):
        id_short_url = ShortURL.objects.get(path='google').pk
        response = self.client.post('/admin/delete/', {'id_short_url': id_short_url})
        self.assertRedirects(response, '/admin/', fetch_redirect_response=False)
        response = self.client.post('/admin/delete/', {'id_short_url': id_short_url})
        self.assertEqual(response.status_code, 400)

    def test_create_short_url(self):
        form_url = reverse('home')
        response = self.client.post(form_url, {
            'redirect': 'http://example.com',
        })
        self.assertRedirects(response, form_url, fetch_redirect_response=False)



'''Shorty Template Tags'''
from django import template
from django.core.urlresolvers import reverse

register = template.Library()


@register.simple_tag(takes_context=True)
def build_short_url(context, path):
    return context['request'].build_absolute_uri(reverse('redirect', kwargs={'slug': path}))



'''Shorty Views'''

from django.shortcuts import render
from django.http import HttpResponseRedirect, HttpResponseBadRequest, Http404
from django.views.decorators.http import require_POST
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse, NoReverseMatch
from django.utils.six.moves.urllib.parse import urlparse
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

from .models import ShortURL
from .forms import ShortURLForm
from .app_settings import EXTERNAL_FLAG, CANONICAL_DOMAIN


def check_initial_redirect(request, path_slug):

    if not CANONICAL_DOMAIN:
        return None

    canonical_scheme, canonical_netloc = urlparse(CANONICAL_DOMAIN)[:2]
    current_scheme, current_netloc = urlparse(request.build_absolute_uri())[:2]
    if not (current_scheme == canonical_scheme and current_netloc == canonical_netloc):
        qs = request.META['QUERY_STRING']
        if qs:
            qs = '?{}'.format(qs)
        return '{}/{}{}'.format(CANONICAL_DOMAIN, path_slug, qs)


def do_redirect(request, slug=None):

    path_slug = slug
    if not path_slug:
        try:
            path_slug = request.META['HTTP_HOST'].split('.', 1)[0]
        except KeyError:
            return HttpResponseBadRequest()

    initial_redirect = check_initial_redirect(request, path_slug)

    if initial_redirect:
        return HttpResponseRedirect(initial_redirect)

    try:
        short_url = ShortURL.objects.get(path=path_slug)
    except ShortURL.DoesNotExist:
        if not slug:
            return HttpResponseRedirect(reverse('home'))
        raise Http404

    if EXTERNAL_FLAG and not short_url.external and not request.user.is_authenticated():
        return login_required(lambda _: None)(request)

    if 'preview' in request.GET or 'p' in request.GET:
        return render(request, 'shorty/preview.html', {
            'url': short_url,
        })

    return HttpResponseRedirect(short_url.redirect)


@login_required
def home(request):

    path = None

    if 'path' in request.session:
        try:
            path = request.build_absolute_uri(reverse('redirect', kwargs={'slug': request.session['path']}))
        except NoReverseMatch:
            pass
        del request.session['path']

    if request.method == 'POST':
        form = ShortURLForm(request, request.POST)
        if form.is_valid():
            model = form.save(commit=False)
            model.user = request.user
            model.save()
            request.session['path'] = model.path
            return HttpResponseRedirect(reverse('home'))
    else:
        form = ShortURLForm(request)

    paginator = Paginator(request.user.short_urls.all().order_by('-created'), 10)

    page = request.GET.get('page')
    try:
        short_urls = paginator.page(page)
    except PageNotAnInteger:
        short_urls = paginator.page(1)
    except EmptyPage:
        short_urls = paginator.page(paginator.num_pages)

    return render(request, 'shorty/home.html', {
        'EXTERNAL_FLAG': EXTERNAL_FLAG,
        'path': path,
        'form': form,
        'short_urls': short_urls,
        'redirect_base': request.build_absolute_uri(reverse('redirect_base')),
    })


@login_required
@require_POST
def delete(request):
    try:
        url = ShortURL.objects.get(pk=request.POST['id_short_url'], user=request.user)
    except (ShortURL.DoesNotExist, KeyError):
        return HttpResponseBadRequest()
    url.delete()
    return HttpResponseRedirect(reverse('home'))




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('shorty', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='shorturl',
            options={'ordering': ('created',)},
        ),
        migrations.AlterField(
            model_name='shorturl',
            name='created',
            field=models.DateTimeField(auto_now_add=True),
            preserve_default=True,
        ),
    ]



DEFAULT_SETTINGS = {
    'AUTOCONFIG_URL_PREFIXES': {
        'shorty': '',
    },
}

SETTINGS = {
    'INSTALLED_APPS': [
        'django.contrib.admin',
        'nuit',
    ],
    'NUIT_GLOBAL_TITLE': 'Shorty',
    'CSP_SCRIPT_SRC': (
        "'self'",
        "https://cdnjs.cloudflare.com/ajax/libs/clipboard.js/1.5.8/clipboard.min.js",
    ),
}
